package GGI::Mode;

=head1 NAME

GGI::Mode - Perl interface to the ggi_mode structure

=head1 SYNOPSIS

  use GGI::Mode;
  
  $mode = new GGI::Mode;

  $frames = $mode->frames;
  $mode->frames($num_frames);
  $virt = $mode->virt;
  $mode->virt->x($width);
  $mode->virt->y($height);
  $visible = $mode->visible;
  $mode->visible->x($width);
  $mode->visible->y($height);
  $dpp = $mode->dpp;
  $mode->dpp->x($width);
  $mode->dpp->y($height);
  $dpp = $mode->size;
  $mode->size->x($width);
  $mode->size->y($height);

  $graphtype = $mode->graphtype;
  $mode->graphtype($graphtype);
  $mode->graphtype($depth, $size, $scheme, $subscheme);

  print "My mode is $mode\n";

=head1 DESCRIPTION

Implements simple methods to access the members of the ggi_mode structure.

=cut

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);
use GGI::Coord;
use GGI::GraphType;
use overload 
    '""' => \&stringify, 
    '0+' => \&boolstatus, 
    'bool' => \&boolstatus;

require Exporter;
require DynaLoader;
require AutoLoader;

=head1 Exported Constants

GGI_AUTO

=cut

@ISA = qw(Exporter AutoLoader DynaLoader);

@EXPORT = qw(GGI_AUTO);

$VERSION = '0.03';

bootstrap GGI::Mode $VERSION;

# Preloaded methods

sub new {
    my ($class) = shift();
    my $this = {};
    $this->{obj} = ggi_mode_malloc();
    $this->{status} = undef;
    $this->{lastused} = undef;
    bless $this;
    return $this;
}

sub frames {
    my $this = shift();
    if (@_ + 0) {
	my($arg) = shift();
	$arg = &GGI_AUTO unless defined($arg);
	return ggi_mode_set_member_frames($this->{obj}, $arg);
    }
    return ggi_mode_get_member_frames($this->{obj});
}

sub visible {
    my $this = shift();
    unless (defined($this->{visible})) {
	$this->{visible} = not_quite_new GGI::Coord ggi_mode_get_member_visible($this->{obj});
    }
    return($this->{visible}) unless @_ + 0;
    my ($arg) = shift();
    if (ref($arg) and UNIVERSAL::isa($arg,'GGI::Coord')) {
	($this->{visible})->x($arg->x);
	($this->{visible})->y($arg->y);
	return $this->{visible};
    }
    ($this->{visible})->x(defined($arg) ? $arg : &GGI_AUTO);
    $arg = shift();
    ($this->{visible})->y(defined($arg) ? $arg : &GGI_AUTO);
    return $this->{visible};
}

sub virt {
    my $this = shift();
    unless (defined($this->{virt})) {
	$this->{virt} = not_quite_new GGI::Coord ggi_mode_get_member_virt($this->{obj});
    }
    return($this->{virt}) unless @_ + 0;
    my ($arg) = shift();
    if (ref($arg) and UNIVERSAL::isa($arg,'GGI::Coord')) {
	($this->{virt})->x($arg->x);
	($this->{virt})->y($arg->y);
	return $this->{virt};
    }
    ($this->{virt})->x(defined($arg) ? $arg : &GGI_AUTO);
    $arg = shift();
    ($this->{virt})->y(defined($arg) ? $arg : &GGI_AUTO);
    return $this->{virt};
}

sub size {
    my $this = shift();
    unless (defined($this->{size})) {
	$this->{size} = not_quite_new GGI::Coord ggi_mode_get_member_size($this->{obj});
    }
    return($this->{size}) unless @_ + 0;
    my ($arg) = shift();
    if (ref($arg) and UNIVERSAL::isa($arg,'GGI::Coord')) {
	($this->{size})->x($arg->x);
	($this->{size})->y($arg->y);
	return $this->{size};
    }
    ($this->{size})->x(defined($arg) ? $arg : &GGI_AUTO);
    $arg = shift();
    ($this->{size})->y(defined($arg) ? $arg : &GGI_AUTO);
    return $this->{size};
}

sub dpp {
    my $this = shift();
    unless (defined($this->{dpp})) {
	$this->{dpp} = not_quite_new GGI::Coord ggi_mode_get_member_dpp($this->{obj});
    }
    return($this->{dpp}) unless @_ + 0;
    my ($arg) = shift();
    if (ref($arg) and UNIVERSAL::isa($arg,'GGI::Coord')) {
	($this->{dpp})->x($arg->x);
	($this->{dpp})->y($arg->y);
	return $this->{dpp};
    }
    ($this->{dpp})->x(defined($arg) ? $arg : &GGI_AUTO);
    $arg = shift();
    ($this->{dpp})->y(defined($arg) ? $arg : &GGI_AUTO);
    return $this->{dpp};
}

sub graphtype {
    my $this = shift();
    unless (defined($this->{graphtype})) {
	$this->{graphtype} = not_quite_new GGI::GraphType ggi_mode_get_member_graphtype($this->{obj});
    }
    my($arg) = shift();
    if(ref($arg) and UNIVERSAL::isa($arg, 'GGI::GraphType')) {
	($this->{graphtype})->graphtype($arg->graphtype);
    }
    elsif (@_ + 0 or defined($arg)) {
	($this->{graphtype})->depth(defined($arg) ? $arg : &GT_AUTO);
	$arg = shift();
	($this->{graphtype})->size(defined($arg) ? $arg : &GT_AUTO);
	$arg = shift();
	($this->{graphtype})->scheme(defined($arg) ? $arg : &GT_AUTO);
	$arg = shift();
	($this->{graphtype})->subscheme(defined($arg) ? $arg : &GT_AUTO);
    }
    return $this->{graphtype};
}

sub status {
    my ($this) = shift();  
    $this->{status} = shift() if (@_ + 0);
    return $this->{status};
}

sub lastused {
    my ($this) = shift();  
    $this->{lastused} = shift() if (@_ + 0);
    return $this->{lastused};
}

sub parse {
    my $this = shift();
    return ggiParseMode(shift(), $this->{obj});
}

sub boolstatus {
    my ($this) = shift();  
    return $this->{status};
}

sub stringify {
    my $this = shift();
    return ("<INVALID MODE> (status " . $this->{status} . ")") unless ($this->{obj});
    my($result) = " " x 256;
    ggiSPrintMode($result, $this->{obj});
    $result =~ s/\s+\Z//;
    return($result);
}

sub DESTROY {
    my $this = shift();
    ggi_mode_free($this->{obj});
    $this->{obj} = undef();
}

sub AUTOLOAD {
    $AutoLoader::AUTOLOAD = $AUTOLOAD;
    goto &AutoLoader::AUTOLOAD;
}

1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GGI::(3pm), perl(1).

=cut


1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GGI::(3pm), perl(1).

=cut
